var basel, quartiere, path, svg, g;

async function drawBasel() {
  await d3.csv("data/migration/OpenDataUmzuege.csv").then(function(data) {
    veranderung = data.map(function(d) {
      return {
        origin_wohnviertel_name: d.origin_wohnviertel_name,
        origin_wohnviertel_id: d.origin_wohnviertel_id,
        jahr: d.jahr,
        destination_wohnviertel_id: d.destination_wohnviertel_id,
        destination_wohnviertel_name: d.destination_wohnviertel_name,
        Count: d.Count
      };
    });
  });
  await d3.json("data/geo/wohnquartiere2.topojson").then(function(topology) {
    console.log("Step 1 - loading GeoJSON data", topology);

    // extract only cantons from the GeoJSON data
    basel = topojson.feature(topology, topology.objects.wohnquartiere2);

    console.log(basel);

    svg = d3
      .select("#world")
      .append("svg")
      .attr("id", "Stadtkarte")
      .attr("width", width)
      .attr("height", height)
      .attr("viewBox", "425, -820, 600, 940");

    path = d3.geoPath().projection(null);

    quartiere = d3
      .select("#Stadtkarte")
      .selectAll("path.wohnquartiere2")
      .data(basel.features)
      .enter()
      .append("path")
      .attr("class", "quartier")
      .attr("d", path)
      .style("stroke", "grey")
      .attr("fill", function(d, i) {
        return colorBS(
          veranderung
            .filter(
              f =>
                f.destination_wohnviertel_name === d.properties.TYPE &&
                f.jahr == year
            )
            .map(a => a.Count)
            .reduce((a, b) => {
              return Number(a) + Number(b);
            }, 0)
        );
      })
      .attr("stroke-width", 1)
      .attr("title", function(d, i) {
        return d.properties.TYPE;
      });
    quartiere.on("mousemove", function(d, i) {
      var mouse = d3.mouse(svg.node()).map(function(d) {
        return parseInt(d);
      });
      tooltip.html(function() {
        var checkbox4 = document.getElementById("hinzu");
        var checkbox5 = document.getElementById("weg");
        var checkbox6 = document.getElementById("alles");
        if (checkbox4.checked == true) {
          return (
            d.properties.TYPE +
            "<br/>" +
            "Zuz&uumlge: " +
            veranderung
              .filter(
                f =>
                  f.destination_wohnviertel_name === d.properties.TYPE &&
                  f.origin_wohnviertel_name != f.destination_wohnviertel_name &&
                  f.jahr == year
              )
              .map(a => a.Count)
              .reduce((a, b) => {
                return Number(a) + Number(b);
              }, 0)
          );
        }
        if (checkbox5.checked == true) {
          return (
            d.properties.TYPE +
            "<br/>" +
            "Wegz&uumlge: " +
            veranderung
              .filter(
                f =>
                  f.origin_wohnviertel_name === d.properties.TYPE &&
                  f.destination_wohnviertel_name != f.origin_wohnviertel_name &&
                  f.jahr == year
              )
              .map(a => a.Count)
              .reduce((a, b) => {
                return Number(a) + Number(b);
              }, 0)
          );
        }
      });
    });
  });
}

var throttleTimer;
function throttle() {
  window.clearTimeout(throttleTimer);
  throttleTimer = window.setTimeout(function() {
    redraw();
  }, 200);
}
