var switzerland, cantons, path, svg, g;
// Load world data
async function drawCH() {
  await d3.csv("data/migration/OpenDataMigration.csv").then(function(data) {
    umzuge = data.map(function(d) {
      return {
        origin: d.origin,
        jahr_id: d.jahr_id,
        destination: d.destination,
        unaggregated_country_name: d.unaggregated_country_name,
        Count: d.Count
      };
    });
    console.log(umzuge[0]);
  });
  await d3.json("data/geo/ch.json").then(function(geojson_switzerland) {
    console.log("Step 1 - loading GeoJSON data", geojson_switzerland);

    // extract only cantons from the GeoJSON data
    switzerland = topojson.feature(
      geojson_switzerland,
      geojson_switzerland.objects.cantons
    );

    console.log(switzerland);

    svg = d3
      .select("#world")
      .append("svg")
      .attr("id", "Schweizerkarte")
      .attr("width", width)
      .attr("height", height)
      .attr("viewBox", "-40, 10, 1050, 400");

    path = d3.geoPath().projection(null);

    cantons = d3
      .select("#Schweizerkarte")
      .selectAll("path.canton")
      .data(switzerland.features)
      .enter()
      .append("path")
      .attr("class", "canton")
      .attr("d", path)
      .attr("fill", function(d, i) {
        return colors(
          umzuge
            .filter(f => f.origin === d.properties.abbr && f.jahr_id == year)
            .map(a => a.Count)
            .reduce((a, b) => {
              return Number(a) + Number(b);
            }, 0)
        );
      })
      .style("stroke", "grey")
      .attr("stroke-width", 1)
      .attr("title", function(d, i) {
        return d.properties.name;
      })
      .attr("name", function(d, i) {
        return d.properties.abbr;
      });

    cantons
      .on("mousemove", function(d, i) {
        var mouse = d3.mouse(svg.node()).map(function(d) {
          return parseInt(d);
        });
        tooltip.classed("hidden", false).html(function() {
          var checkbox4 = document.getElementById("hinzu");
          var checkbox5 = document.getElementById("weg");
          var checkbox6 = document.getElementById("alles");
          if (checkbox4.checked == true && d.properties.abbr !== "BS") {
            return (
              d.properties.name +
              "<br/>" +
              umzuge
                .filter(
                  f => f.origin === d.properties.abbr && f.jahr_id == year
                )
                .map(a => a.Count)
                .reduce((a, b) => {
                  return Number(a) + Number(b);
                }, 0) +
              " nach Basel-Stadt"
            );
          }
          if (checkbox4.checked == true && d.properties.abbr == "BS") {
            return (
              "Gesamt" +
              "<br/>" +
              umzuge
                .filter(
                  f => f.destination === d.properties.abbr && f.jahr_id == year
                )
                .map(a => a.Count)
                .reduce((a, b) => {
                  return Number(a) + Number(b);
                }, 0) +
              " nach Basel-Stadt"
            );
          }
          if (checkbox5.checked == true && d.properties.abbr !== "BS") {
            return (
              d.properties.name +
              "<br/>" +
              umzuge
                .filter(
                  f => f.destination === d.properties.abbr && f.jahr_id == year
                )
                .map(a => a.Count)
                .reduce((a, b) => {
                  return Number(a) + Number(b);
                }, 0) +
              " von Basel-Stadt"
            );
          }
          if (checkbox5.checked == true && d.properties.abbr == "BS") {
            return (
              "Gesamt" +
              "<br/>" +
              umzuge
                .filter(
                  f => f.origin === d.properties.abbr && f.jahr_id == year
                )
                .map(a => a.Count)
                .reduce((a, b) => {
                  return Number(a) + Number(b);
                }, 0) +
              " von Basel-Stadt weg"
            );
          }
        });
      })
      .on("mouseout", function(d, i) {
        tooltip.classed("hidden", true);
      });
  });
}
