//Chart Basel Stadt

async function chartBS() {
  await d3.csv("data/migration/OpenDataUmzuege.csv").then(function(data) {
    const filtered = data.filter(f => f.jahr == year);

    const table = {
      name: "Gesamtzahl",
      children: d3
        .nest()
        .key(function(d) {
          return d.origin_wohnviertel_name;
        })
        .key(function(d) {
          return d.destination_wohnviertel_name;
        })
        .key(function(d) {
          return d.destination_bezirk_name;
        })
        .entries(filtered)
        .map(function(d, index0, array0) {
          return {
            name: d.key,
            index: (index0 + 1) / array0.length,
            children: d.values.map((value, index1, array1) => ({
              name: value.key,
              index:
                index0 / array0.length +
                (1 + index1) / (array0.length * array1.length),
              children: value.values.map((valued, index2, array2) => ({
                name: valued.key,
                index:
                  index0 / array0.length +
                  index1 / (array0.length * array1.length) +
                  index2 / (array0.length * array1.length * array2.length),
                Count: valued.values
                  .map(a => Number(a.Count))
                  .reduce((a, b) => a + b, 0)
              }))
            }))
          };
        })
    };
    var root = d3.hierarchy(table);
    root.sum(d => Number(d.Count));

    console.log(root);

    const slice = svgs.selectAll("g.slice").data(
      partition(root)
        .descendants()
        .slice(1)
    );

    slice.exit().remove();

    const newSlice = slice
      .enter()
      .append("g")
      .attr("class", "slice")
      .on("click", d => {
        d3.event.stopPropagation();
        focusOn(d);
      });

    newSlice
      .append("title")
      .text(d => d.data.name + "\n" + formatNumber(d.value));

    newSlice
      .append("path")
      .attr("class", "main-arc")
      .style("fill", d => color(d.data.index))
      .attr("d", arc)
      .style("stroke", "#fff")
      .style("stroke-width", 0.2);

    newSlice
      .append("path")
      .attr("class", "hidden-arc")
      .attr("id", (_, i) => `hiddenArc${i}`)
      .style("fill", "none")
      .attr("d", middleArcLine);

    const texts = newSlice
      .append("text")
      .attr("display", d => (textFits(d) ? null : "none"));

    // Add white contour
    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name)
      .style("fill", "none")
      .style("stroke", "#fff")
      .style("stroke-width", 5)
      .style("stroke-linejoin", "round");

    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name);
  });
}

//Chart Zuwanderung Basel-Stadt

async function chartCHZuwanderung() {
  await d3.csv("data/migration/OpenDataMigration.csv").then(function(data) {
    const filter = data.filter(
      f => f.jahr_id == year && f.originregion_name === "Übrige Schweiz"
    );

    const tables = {
      name: "Gesamtzahl",
      children: d3
        .nest()
        .key(function(d) {
          return d.origin;
        })
        .key(function(d) {
          return d.destination_name;
        })
        .entries(filter)
        .map(function(d, index0, array0) {
          return {
            name: d.key,
            index: (index0 + 1) / array0.length,
            children: d.values.map((value, index1, array1) => ({
              name: value.key,
              index:
                index0 / array0.length +
                (1 + index1) / (array0.length * array1.length),

              Count: value.values
                .map(a => Number(a.Count))
                .reduce((a, b) => a + b, 0)
            }))
          };
        })
    };

    var root = d3.hierarchy(tables);
    root.sum(d => Number(d.Count));

    const slice = svgs.selectAll("g.slice").data(
      partition(root)
        .descendants()
        .slice(1)
    );
    console.log(root);
    slice.exit().remove();

    const newSlice = slice
      .enter()
      .append("g")
      .attr("class", "slice")
      .on("click", d => {
        d3.event.stopPropagation();
        focusOn(d);
      });

    newSlice
      .append("title")
      .text(d => d.data.name + "\n" + formatNumber(d.value));

    newSlice
      .append("path")
      .attr("class", "main-arc")
      .style("fill", d => color(d.data.index))
      .attr("d", arc)
      .style("stroke", "#fff")
      .style("stroke-width", 0.2);

    newSlice
      .append("path")
      .attr("class", "hidden-arc")
      .attr("id", (_, i) => `hiddenArc${i}`)
      .style("fill", "none")
      .attr("d", middleArcLine);

    const texts = newSlice
      .append("text")
      .attr("display", d => (textFits(d) ? null : "none"));

    // Add white contour
    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name)
      .style("fill", "none")
      .style("stroke", "#fff")
      .style("stroke-width", 5)
      .style("stroke-linejoin", "round");

    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name);
  });
}
function focusOn(d = { x0: 0, x1: 1, y0: 0, y1: 1 }) {
  // Reset to top-level if no data point specified

  const transitions = svgs
    .transition()
    .duration(750)
    .tween("scale", () => {
      const xd = d3.interpolate(x.domain(), [d.x0, d.x1]),
        yd = d3.interpolate(y.domain(), [d.y0, 1]);
      return t => {
        x.domain(xd(t));
        y.domain(yd(t));
      };
    });

  transitions.selectAll("path.main-arc").attrTween("d", d => () => arc(d));

  transitions
    .selectAll("path.hidden-arc")
    .attrTween("d", d => () => middleArcLine(d));

  transitions
    .selectAll("text")
    .attrTween("display", d => () => (textFits(d) ? null : "none"));

  moveStackToFront(d);

  //

  function moveStackToFront(elD) {
    svgs
      .selectAll(".slice")
      .filter(d => d === elD)
      .each(function(d) {
        this.parentNode.appendChild(this);
        if (d.parent) {
          moveStackToFront(d.parent);
        }
      });
  }
}

//Chart Abwanderung aus Basel

async function chartCHAbwanderung() {
  await d3.csv("data/migration/OpenDataMigration.csv").then(function(data) {
    const filter = data.filter(
      f => f.jahr_id == year && f.destinationregion_name === "Übrige Schweiz"
    );

    const tables = {
      name: "Gesamtzahl",
      children: d3
        .nest()
        .key(function(d) {
          return d.origin_name;
        })
        .key(function(d) {
          return d.destination;
        })
        .entries(filter)
        .map(function(d, index0, array0) {
          return {
            name: d.key,
            index: (index0 + 1) / array0.length,
            children: d.values.map((value, index1, array1) => ({
              name: value.key,
              index:
                index0 / array0.length +
                (1 + index1) / (array0.length * array1.length),
              Count: value.values
                .map(a => Number(a.Count))
                .reduce((a, b) => a + b, 0)
            }))
          };
        })
    };

    var root = d3.hierarchy(tables);
    root.sum(d => Number(d.Count));

    const slice = svgs.selectAll("g.slice").data(
      partition(root)
        .descendants()
        .slice(1)
    );
    console.log(root);
    slice.exit().remove();

    const newSlice = slice
      .enter()
      .append("g")
      .attr("class", "slice")
      .on("click", d => {
        d3.event.stopPropagation();
        focusOn(d);
      });

    newSlice
      .append("title")
      .text(d => d.data.name + "\n" + formatNumber(d.value));

    newSlice
      .append("path")
      .attr("class", "main-arc")
      .style("fill", d => color(d.data.index))
      .attr("d", arc)
      .style("stroke", "#fff")
      .style("stroke-width", 0.2);

    newSlice
      .append("path")
      .attr("class", "hidden-arc")
      .attr("id", (_, i) => `hiddenArc${i}`)
      .style("fill", "none")
      .attr("d", middleArcLine);

    const texts = newSlice
      .append("text")
      .attr("display", d => (textFits(d) ? null : "none"));

    // Add white contour
    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name)
      .style("fill", "none")
      .style("stroke", "#fff")
      .style("stroke-width", 5)
      .style("stroke-linejoin", "round");

    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name);
  });
}
function focusOn(d = { x0: 0, x1: 1, y0: 0, y1: 1 }) {
  // Reset to top-level if no data point specified

  const transitions = svgs
    .transition()
    .duration(750)
    .tween("scale", () => {
      const xd = d3.interpolate(x.domain(), [d.x0, d.x1]),
        yd = d3.interpolate(y.domain(), [d.y0, 1]);
      return t => {
        x.domain(xd(t));
        y.domain(yd(t));
      };
    });

  transitions.selectAll("path.main-arc").attrTween("d", d => () => arc(d));

  transitions
    .selectAll("path.hidden-arc")
    .attrTween("d", d => () => middleArcLine(d));

  transitions
    .selectAll("text")
    .attrTween("display", d => () => (textFits(d) ? null : "none"));

  moveStackToFront(d);

  //

  function moveStackToFront(elD) {
    svgs
      .selectAll(".slice")
      .filter(d => d === elD)
      .each(function(d) {
        this.parentNode.appendChild(this);
        if (d.parent) {
          moveStackToFront(d.parent);
        }
      });
  }
}

//Chart Zuwanderung aus der Welt

async function chartWorldZuwanderung() {
  await d3.csv("data/migration/OpenDataMigration.csv").then(function(data) {
    const filter = data.filter(
      f =>
        f.jahr_id == year &&
        f.originregion_name !== "Übrige Schweiz" &&
        f.origin !== "BS" &&
        f.origin !== "unbekannt" &&
        f.destination !== "unbekannt" &&
        f.destinationregion_name === "Basel-Stadt"
    );

    const tables = {
      name: "Gesamtzahl",
      children: d3
        .nest()
        .key(function(d) {
          return d.originregion_name;
        })
        .key(function(d) {
          return d.origin;
        })
        .key(function(d) {
          return d.destination_name;
        })
        .entries(filter)
        .map(function(d, index0, array0) {
          return {
            name: d.key,
            index: (index0 + 1) / array0.length,
            children: d.values.map((value, index1, array1) => ({
              name: value.key,
              index:
                index0 / array0.length +
                (1 + index1) / (array0.length * array1.length),
              children: value.values.map((valued, index2, array2) => ({
                name: valued.key,
                index:
                  index0 / array0.length +
                  index1 / (array0.length * array1.length) +
                  index2 / (array0.length * array1.length * array2.length),
                Count: valued.values
                  .map(a => Number(a.Count))
                  .reduce((a, b) => a + b, 0)
              }))
            }))
          };
        })
    };

    var root = d3.hierarchy(tables);
    root.sum(d => Number(d.Count));

    const slice = svgs.selectAll("g.slice").data(
      partition(root)
        .descendants()
        .slice(1)
    );
    console.log(root);
    slice.exit().remove();

    const newSlice = slice
      .enter()
      .append("g")
      .attr("class", "slice")
      .on("click", d => {
        d3.event.stopPropagation();
        focusOn(d);
      });

    newSlice
      .append("title")
      .text(d => d.data.name + "\n" + formatNumber(d.value));

    newSlice
      .append("path")
      .attr("class", "main-arc")
      .style("fill", d => color(d.data.index))
      .attr("d", arc)
      .style("stroke", "#fff")
      .style("stroke-width", 0.2);

    newSlice
      .append("path")
      .attr("class", "hidden-arc")
      .attr("id", (_, i) => `hiddenArc${i}`)
      .style("fill", "none")
      .attr("d", middleArcLine);

    const texts = newSlice
      .append("text")
      .attr("display", d => (textFits(d) ? null : "none"));

    // Add white contour
    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name)
      .style("fill", "none")
      .style("stroke", "#fff")
      .style("stroke-width", 5)
      .style("stroke-linejoin", "round");

    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name);
  });
}
function focusOn(d = { x0: 0, x1: 1, y0: 0, y1: 1 }) {
  // Reset to top-level if no data point specified

  const transitions = svgs
    .transition()
    .duration(750)
    .tween("scale", () => {
      const xd = d3.interpolate(x.domain(), [d.x0, d.x1]),
        yd = d3.interpolate(y.domain(), [d.y0, 1]);
      return t => {
        x.domain(xd(t));
        y.domain(yd(t));
      };
    });

  transitions.selectAll("path.main-arc").attrTween("d", d => () => arc(d));

  transitions
    .selectAll("path.hidden-arc")
    .attrTween("d", d => () => middleArcLine(d));

  transitions
    .selectAll("text")
    .attrTween("display", d => () => (textFits(d) ? null : "none"));

  moveStackToFront(d);

  //

  function moveStackToFront(elD) {
    svgs
      .selectAll(".slice")
      .filter(d => d === elD)
      .each(function(d) {
        this.parentNode.appendChild(this);
        if (d.parent) {
          moveStackToFront(d.parent);
        }
      });
  }
}

//Chart Abwanderung in die Welt
async function chartWorldAbwanderung() {
  await d3.csv("data/migration/OpenDataMigration.csv").then(function(data) {
    const filter = data.filter(
      f =>
        f.jahr_id == year &&
        f.originregion_name === "Basel-Stadt" &&
        f.origin !== "BS" &&
        f.origin !== "unbekannt" &&
        f.destination !== "unbekannt" &&
        f.destinationregion_name !== "Übrige Schweiz"
    );

    const tables = {
      name: "Gesamtzahl",
      children: d3
        .nest()
        .key(function(d) {
          return d.origin_name;
        })
        .key(function(d) {
          return d.destinationregion_name;
        })
        .key(function(d) {
          return d.destination;
        })
        .entries(filter)
        .map(function(d, index0, array0) {
          return {
            name: d.key,
            index: (index0 + 1) / array0.length,
            children: d.values.map((value, index1, array1) => ({
              name: value.key,
              index:
                index0 / array0.length +
                (1 + index1) / (array0.length * array1.length),
              children: value.values.map((valued, index2, array2) => ({
                name: valued.key,
                index:
                  index0 / array0.length +
                  index1 / (array0.length * array1.length) +
                  index2 / (array0.length * array1.length * array2.length),
                Count: valued.values
                  .map(a => Number(a.Count))
                  .reduce((a, b) => a + b, 0)
              }))
            }))
          };
        })
    };

    var root = d3.hierarchy(tables);
    root.sum(d => Number(d.Count));

    const slice = svgs.selectAll("g.slice").data(
      partition(root)
        .descendants()
        .slice(1)
    );
    console.log(root);
    slice.exit().remove();

    const newSlice = slice
      .enter()
      .append("g")
      .attr("class", "slice")
      .on("click", d => {
        d3.event.stopPropagation();
        focusOn(d);
      });

    newSlice
      .append("title")
      .text(d => d.data.name + "\n" + formatNumber(d.value));

    newSlice
      .append("path")
      .attr("class", "main-arc")
      .style("fill", d => color(d.data.index))
      .attr("d", arc)
      .style("stroke", "#fff")
      .style("stroke-width", 0.2);

    newSlice
      .append("path")
      .attr("class", "hidden-arc")
      .attr("id", (_, i) => `hiddenArc${i}`)
      .style("fill", "none")
      .attr("d", middleArcLine);

    const texts = newSlice
      .append("text")
      .attr("display", d => (textFits(d) ? null : "none"));

    // Add white contour
    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name)
      .style("fill", "none")
      .style("stroke", "#fff")
      .style("stroke-width", 5)
      .style("stroke-linejoin", "round");

    texts
      .append("textPath")
      .attr("startOffset", "35%")
      .attr("xlink:href", (_, i) => `#hiddenArc${i}`)
      .text(d => d.data.name);
  });
}
function focusOn(d = { x0: 0, x1: 1, y0: 0, y1: 1 }) {
  // Reset to top-level if no data point specified

  const transitions = svgs
    .transition()
    .duration(750)
    .tween("scale", () => {
      const xd = d3.interpolate(x.domain(), [d.x0, d.x1]),
        yd = d3.interpolate(y.domain(), [d.y0, 1]);
      return t => {
        x.domain(xd(t));
        y.domain(yd(t));
      };
    });

  transitions.selectAll("path.main-arc").attrTween("d", d => () => arc(d));

  transitions
    .selectAll("path.hidden-arc")
    .attrTween("d", d => () => middleArcLine(d));

  transitions
    .selectAll("text")
    .attrTween("display", d => () => (textFits(d) ? null : "none"));

  moveStackToFront(d);

  //

  function moveStackToFront(elD) {
    svgs
      .selectAll(".slice")
      .filter(d => d === elD)
      .each(function(d) {
        this.parentNode.appendChild(this);
        if (d.parent) {
          moveStackToFront(d.parent);
        }
      });
  }
}
